// Simple ARM test program for disassembler verification
// This program performs basic arithmetic operations

int main() {
    int a = 10;
    int b = 20;
    int c = 0;
    
    // Perform arithmetic operations
    c = a + b;      // c = 30
    c = c - b;      // c = 10
    c = c + 5;      // c = 15
    
    return c;
}

