import { describe, test, expect } from "vitest";

/*
let services: ReturnType<typeof createIsaServices>;
let parse:    ReturnType<typeof parseHelper<ISASpec>>;
let document: LangiumDocument<ISASpec> | undefined;

beforeAll(async () => {
    services = createIsaServices(EmptyFileSystem);
    parse = parseHelper<ISASpec>(services.Isa);

    // activate the following if your linking test requires elements from a built-in library, for example
    // await services.shared.workspace.WorkspaceManager.initializeWorkspace([]);
});

afterEach(async () => {
    document && clearDocuments(services.shared, [ document ]);
});
*/

describe('Linking tests', () => {

    test('Placeholder - TODO: Add linking tests', () => {
        // TODO: Add linking tests
        expect(true).toBe(true);
    });
});
